#ifndef _L4_SYSCALLS_H
# error "Never use <l4/bits/syscalls.h> directly; include <l4/syscalls.h> instead."
#endif

#ifndef _L4_EXTERN_INLINE
#define _L4_EXTERN_INLINE extern __inline
#endif


/* Return the pointer to the kernel interface page.  Return the API
   version in API_VERSION if not a null pointer.  Return the API flags
   in API_FLAGS if not a null pointer.  Return the kernel ID in
   KERNEL_ID if not a null pointer.  */
_L4_EXTERN_INLINE l4_kip_t
l4_kernel_interface (l4_api_version_t *api_version, l4_api_flags_t *api_flags,
		     l4_kernel_id_t *kernel_id) __attribute__ ((__pure__));

_L4_EXTERN_INLINE l4_kip_t
l4_kernel_interface (l4_api_version_t *api_version, l4_api_flags_t *api_flags,
		     l4_kernel_id_t *kernel_id)
{
  void *kip;
  l4_word_t version;
  l4_word_t flags;
  l4_word_t id;

  /* The KernelInterface system call is invoked by "lock; nop" and
     returns a pointer to the kernel interface page in %eax, the API
     version in %ecx, the API flags in %edx, and the kernel ID in
     %esi.  */
  asm ("  lock; nop\n" :
       "=a" (kip), "=c" (version), "=d" (flags), "=S" (id));

  if (api_version)
    api_version->raw = version;
  if (api_flags)
    api_flags->raw = flags;
  if (kernel_id)
    kernel_id->raw = id;

  return kip;
}
