#ifndef _L4_KIP_H
#define _L4_KIP_H	1

#include <l4/types.h>

/* Utility macros for structure definitions.  */
#if L4_WORDSIZE == L4_WORDSIZE_32
#define _L4_BITFIELD(name,nr32,nr64) l4_word_t name : nr32
#define _L4_BITFIELD_32(name,nr) l4_word_t name : nr
#define _L4_BITFIELD_64(name,nr)
#else
#if L4_WORDSIZE == L4_WORDSIZE_64
#define _L4_BITFIELD(name,nr32,nr64) l4_word_t name : nr64
#define _L4_BITFIELD_32(name,nr)
#define _L4_BITFIELD_64(name,nr) l4_word_t name : nr
#else
#error "Unsupported word size."
#endif
#endif

#define _L4_WORD_STRUCT(x)	\
  union				\
  {				\
    l4_word_t raw;		\
    struct			\
      x;			\
  }


typedef _L4_WORD_STRUCT 
({
  l4_word_t __pad1 : 16;

  /* The subversion or revision.  */
 l4_word_t subversion : 8;

  /* The interface version.  */
  l4_word_t version : 8;

  _L4_BITFIELD_64 (__pad2, 32);
}) l4_api_version_t;


#define L4_API_FLAGS_LITTLE_ENDIAN	0x0
#define L4_API_FLAGS_BIG_ENDIAN		0x1
#define L4_API_FLAGS_WORDSIZE_32	0x0
#define L4_API_FLAGS_WORDSIZE_64	0x1

typedef _L4_WORD_STRUCT
({
  /* The endianess.  */
  l4_word_t endian : 2;

  /* The word size.  */
  l4_word_t wordsize : 2;

  _L4_BITFIELD (__pad, 28, 60);
}) l4_api_flags_t;


typedef _L4_WORD_STRUCT
({
  l4_word_t __pad : 16;

  /* The kernel sub ID.  */
  l4_word_t subid : 8;

  /* The kernel ID.  */
  l4_word_t id : 8;

  _L4_BITFIELD_64 (__pad, 32);
}) l4_kernel_id_t;


typedef struct
{
  char magic[4];
#if L4_WORDSIZE == L4_WORDSIZE_64
  char __pad1[4];
#endif

  l4_api_version_t api_version;
  l4_api_flags_t api_flags_t;

  l4_word_t kern_desc_ptr;

  l4_word_t __pad2[17];

  _L4_WORD_STRUCT
  ({
    /* Number of memory descriptors.  */
    _L4_BITFIELD (nr, 16, 32);

    /* Offset (in bytes) of memory descriptors in KIP.  */
    _L4_BITFIELD (mem_desc_ptr, 16, 32);
  }) memory_info;

  l4_word_t __pad3[20];

  _L4_WORD_STRUCT
  ({
    /* UTCB size multiplier.  Size of one UTCB block is m * 2^log2_align.  */
    l4_word_t size_mul : 10;

    /* UTCB alignment requirement (must be aligned to 2^log2_align).  */
    l4_word_t log2_align : 4;

    /* Minimal size for UTCB area is 2^log2_min_size.  */
    l4_word_t log2_min_size : 6;

    _L4_BITFIELD (__pad, 12, 44);
  }) utcb_info;

  _L4_WORD_STRUCT
  ({
    /* The size of the KIP area is 2^log2_size.  */
    l4_word_t log2_size : 6;

    _L4_BITFIELD (__pad, 26, 58);
  }) kip_area_info;

  l4_word_t __pad4[2];

  l4_word_t boot_info;

  /* Offset (in bytes) of processor descriptors in KIP.  */
  l4_word_t proc_desc_ptr;

  _L4_WORD_STRUCT
  ({
    /* Minimal time difference that can be read with the system clock
       syscall.  */
    l4_word_t read_precision : 16;

    /* Maximal jitter for a scheduled thread activation.  */
    l4_word_t schedule_precision : 16;

    _L4_BITFIELD_64 (__pad, 32);
  }) clock_info;

  _L4_WORD_STRUCT
  ({
    /* Number of valid thread number bits.  */
    l4_word_t log2_max_thread : 8;

    /* Lowest thread number used for system threads.  */
    l4_word_t system_base : 12;

    /* Lowest thread number available for user threads.  */
    l4_word_t user_base : 12;

    _L4_BITFIELD_64 (__pad, 32);
  }) thread_info;

  _L4_WORD_STRUCT
  ({
    /* Execute access right can be independently set.  */
    l4_word_t execute : 1;

    /* Write access right can be independently set.  */
    l4_word_t write : 1;

    /* Read access right can be independently set.  */
    l4_word_t read : 1;

    l4_word_t __pad : 7;

    /* Page size of 2^(k + 10) is supported by hardware and kernel if
       bit k is set.  */
    _L4_BITFIELD (page_size_mask, 22, 54);
  }) page_info;

  _L4_WORD_STRUCT
  ({
    /* The number of processors minus 1.  */
    l4_word_t processors : 16;

    _L4_BITFIELD (__pad, 12, 44);

    /* The size of one processor description is 2^log2_size.  */
    l4_word_t log2_size : 4;
  }) processor_info;

  /* Privileged system call links.  */
  l4_word_t space_control;
  l4_word_t thread_control;
  l4_word_t processor_control;
  l4_word_t memory_control;

  /* Normal system call links.  */
  l4_word_t ipc;
  l4_word_t lipc;
  l4_word_t unmap;
  l4_word_t exchange_registers;
  l4_word_t system_clock;
  l4_word_t thread_switch;
  l4_word_t schedule;
} *l4_kip_t;

typedef struct
{
#define L4_MEMDESC_UNDEFINED	0x0
#define L4_MEMDESC_CONVENTIONAL	0x1
#define L4_MEMDESC_RESERVED	0x2
#define L4_MEMDESC_DEDICATED	0x3
#define L4_MEMDESC_SHARED	0x4
#define L4_MEMDESC_BOOTLOADER	0xe
#define L4_MEMDESC_ARCH		0xf
  /* The type of the memory descriptor.  */
  l4_word_t type : 4;

  /* The subtype of the memory descriptor if type is
     L4_MEMDESC_BOOTLOADER or L4_MEMDESC_ARCH, otherwise
     undefined.  */
  l4_word_t subtype : 4;

  l4_word_t __pad1 : 1;

  /* 1 if memory is virtual, 0 if it is physical.  */
  l4_word_t virtual : 1;

  _L4_BITFIELD (low, 22, 54);

  l4_word_t __pad2 : 10;

  _L4_BITFIELD (high, 22, 54);
} *l4_memory_desc_t;


typedef struct
{
  /* External frequency in kHz.  */
  l4_word_t external_freq;

  /* Internal frequency in kHz.  */
  l4_word_t internal_freq;

  l4_word_t __pad[2];
} *l4_proc_desc_t;


typedef struct
{
  /* Kernel ID.  */
  l4_kernel_id_t id;

  /* The kernel generation date.  */
  _L4_WORD_STRUCT
  ({
    l4_word_t day : 5;
    l4_word_t month : 4;
    /* The year from 2000 on.  */
    l4_word_t year : 7;

    _L4_BITFIELD (__pad, 16, 48);
  }) gen_date;

  /* The kernel version.  */
  _L4_WORD_STRUCT
  ({
    l4_word_t subsubver : 16;
    l4_word_t subver : 8;
    l4_word_t ver : 8;

    _L4_BITFIELD_64 (__pad, 32);
  }) version;

  char supplier[4];
#if L4_WORDSIZE == L4_WORDSIZE_64
  char __pad[4];
#endif

  /* The kernel version string followed by architecture specific
     feature strings.  */
  char version_parts[0];
} *l4_kern_desc_t;

#endif	/* l4/kip.h */
